@@ -4,15 +4,13 @@ This module contains widgets and GUI
 functions for authenticating users.
 """
 #================================================================
-# $Source: /home/ncq/Projekte/cvs2git/vcs-mirror/gnumed/gnumed/client/wxpython/gmAuthWidgets.py,v $
-# $Id: gmAuthWidgets.py,v 1.45 2009-11-06 15:16:21 ncq Exp $
 __version__ = "$Revision: 1.45 $"
 __author__ = "karsten.hilbert@gmx.net, H.Herb, H.Berger, R.Terry"
 __license__ = "GPL (details at http://www.gnu.org)"
 
 
 # stdlib
-import sys, os.path, cPickle, zlib, logging
+import sys, os.path, cPickle, zlib, logging, re as regex
 
 
 # 3rd party
@@ -143,17 +141,36 @@ def connect_to_database(max_attempts=3, expected_version=None, require_version=T
 			attempt += 1
 			_log.error(u"login attempt failed: %s", e)
 			if attempt < max_attempts:
-				gmGuiHelpers.gm_show_error (_(
-					"Unable to connect to database:\n\n"
-					"%s\n\n"
-					"Please retry with proper credentials or cancel.\n"
-					"\n"
-					'You may also need to check the PostgreSQL client\n'
-					'authentication configuration in pg_hba.conf. For\n'
-					'details see:\n'
-					'\n'
-					'wiki.gnumed.de/bin/view/Gnumed/ConfigurePostgreSQL'
-					) % e,
+				if (u'host=127.0.0.1' in (u'%s' % e)) or (u'host=' not in (u'%s' % e)):
+					msg = _(
+						'Unable to connect to database:\n\n'
+						'%s\n\n'
+						"Are you sure you have got a local database installed ?\n"
+						'\n'
+						"Please retry with proper credentials or cancel.\n"
+						'\n'
+						'You may also need to check the PostgreSQL client\n'
+						'authentication configuration in pg_hba.conf. For\n'
+						'details see:\n'
+						'\n'
+						'wiki.gnumed.de/bin/view/Gnumed/ConfigurePostgreSQL'
+					)
+				else:
+					msg = _(
+						"Unable to connect to database:\n\n"
+						"%s\n\n"
+						"Please retry with proper credentials or cancel.\n"
+						"\n"
+						'You may also need to check the PostgreSQL client\n'
+						'authentication configuration in pg_hba.conf. For\n'
+						'details see:\n'
+						'\n'
+						'wiki.gnumed.de/bin/view/Gnumed/ConfigurePostgreSQL'
+					)
+				msg = msg % e
+				msg = regex.sub(r'password=[^\s]+', u'password=%s' % gmTools.u_replacement_character, msg)
+				gmGuiHelpers.gm_show_error (
+					msg,
 					_('Connecting to backend')
 				)
 			del e
@@ -161,11 +178,14 @@ def connect_to_database(max_attempts=3, expected_version=None, require_version=T
 
 		except gmPG2.dbapi.OperationalError, e:
 			_log.error(u"login attempt failed: %s", e)
-			gmGuiHelpers.gm_show_error (_(
-					"Unable to connect to database:\n\n"
-					"%s\n\n"
-					"Please retry another backend / user / password combination !\n"
-				) % gmPG2.extract_msg_from_pg_exception(e),
+			msg = _(
+				"Unable to connect to database:\n\n"
+				"%s\n\n"
+				"Please retry another backend / user / password combination !\n"
+			) % gmPG2.extract_msg_from_pg_exception(e)
+			msg = regex.sub(r'password=[^\s]+', u'password=%s' % gmTools.u_replacement_character, msg)
+			gmGuiHelpers.gm_show_error (
+				msg,
 				_('Connecting to backend')
 			)
 			del e
@@ -191,6 +211,7 @@ def connect_to_database(max_attempts=3, expected_version=None, require_version=T
 			)
 			if require_version:
 				gmGuiHelpers.gm_show_error(msg + msg_fail, _('Verifying database version'))
+				connected = False
 				continue
 			gmGuiHelpers.gm_show_info(msg + msg_override, _('Verifying database version'))
 
@@ -203,12 +224,14 @@ def connect_to_database(max_attempts=3, expected_version=None, require_version=T
 				gmGuiHelpers.gm_show_warning(msg_time_skew_warn % max_skew, _('Verifying database settings'))
 			else:
 				gmGuiHelpers.gm_show_error(msg_time_skew_fail % max_skew, _('Verifying database settings'))
+				connected = False
 				continue
 
 		sanity_level, message = gmPG2.sanity_check_database_settings()
 		if sanity_level != 0:
 			gmGuiHelpers.gm_show_error((msg_insanity % message), _('Verifying database settings'))
 			if sanity_level == 2:
+				connected = False
 				continue
 
 		gmExceptionHandlingWidgets.set_is_public_database(login.public_db)
@@ -704,6 +727,15 @@ For assistance on using GNUmed please contact:
 #----------------------------------------------------------------
 if __name__ == "__main__":
 
+	if len(sys.argv) < 2:
+		sys.exit()
+
+	if sys.argv[1] != 'test':
+		sys.exit()
+
+	# we don't have tests yet
+	sys.exit()
+
 	from Gnumed.pycommon import gmI18N
 
 	logging.basicConfig(level = logging.DEBUG)
@@ -727,498 +759,6 @@ if __name__ == "__main__":
 			wx.MessageBox(_("You tried to log in as [%s] with password [%s].\nHost:%s, DB: %s, Port: %s") % (lp.GetUser(),lp.GetPassword(),lp.GetHost(),lp.GetDatabase(),lp.GetPort()))
 		dlg.Destroy()
 #		app.MainLoop()
-	#-----------------------------------------------
-	if len(sys.argv) > 1 and sys.argv[1] == 'test':
-		print "no regression tests yet"
 
 #================================================================
-# $Log: gmAuthWidgets.py,v $
-# Revision 1.45  2009-11-06 15:16:21  ncq
-# - add missing dot
-#
-# Revision 1.44  2009/10/20 10:25:10  ncq
-# - try not to log previous exception instances
-#
-# Revision 1.43  2009/09/23 14:33:14  ncq
-# - improved failed-auth message
-#
-# Revision 1.42  2009/07/02 20:50:04  ncq
-# - cleanup
-#
-# Revision 1.41  2009/07/01 17:06:33  ncq
-# - improved wording
-#
-# Revision 1.40  2009/06/11 13:04:13  ncq
-# - cleanup
-#
-# Revision 1.39  2009/06/11 11:07:57  ncq
-# - properly display localhost
-#
-# Revision 1.38  2009/04/14 11:02:20  ncq
-# - if not debugging and not in development sort out database
-#   profiles with incompatible names, this essentially hardcodes
-#   the database name for the first time
-#
-# Revision 1.37  2009/04/03 10:40:17  ncq
-# - calculate current db name from expected version
-#
-# Revision 1.36  2009/04/03 09:38:20  ncq
-# - bump db version
-#
-# Revision 1.35  2009/02/05 13:03:08  ncq
-# - cleanup
-#
-# Revision 1.34  2009/01/15 11:35:41  ncq
-# - cleanup
-#
-# Revision 1.33  2008/12/25 17:46:09  ncq
-# - better logging
-# - work around PG unknown initial encoding problem
-#
-# Revision 1.32  2008/11/20 18:48:15  ncq
-# - add default help desk to builtin default backend profile
-#
-# Revision 1.31  2008/10/22 12:13:09  ncq
-# - bump db version
-#
-# Revision 1.30  2008/10/12 16:06:07  ncq
-# - make profile label show db@host
-# - log profile used
-#
-# Revision 1.29  2008/09/09 20:18:51  ncq
-# - default to any-doc if no previous logins found
-#
-# Revision 1.28  2008/08/31 16:47:48  ncq
-# - proper policy when retrieving previous logins
-#
-# Revision 1.27  2008/08/31 16:34:12  ncq
-# - properly set src_order and policy when retrieving profile *data*
-#
-# Revision 1.26  2008/08/31 16:15:02  ncq
-# - use "extend" policy when retrieving backend profiles
-#
-# Revision 1.25  2008/07/30 12:50:34  ncq
-# - cleanup
-#
-# Revision 1.24  2008/07/16 11:11:10  ncq
-# - set_helpdesk (no more _database_)
-#
-# Revision 1.23  2008/07/10 20:52:21  ncq
-# - better to call path detection with app_name and wx
-#
-# Revision 1.22  2008/07/07 11:35:19  ncq
-# - keyboard shortcuts for check boxes
-#
-# Revision 1.21  2008/05/28 21:26:40  ncq
-# - gracefully handle OperationalErrors such as "server not available"
-#
-# Revision 1.20  2008/05/13 14:10:35  ncq
-# - be more permissive in time skew check
-# - handle per-profile public-db/helpdesk options
-#
-# Revision 1.19  2008/04/16 20:39:39  ncq
-# - working versions of the wxGlade code and use it, too
-# - show client version in login dialog
-#
-# Revision 1.18  2008/03/20 15:30:21  ncq
-# - use gmPG2.sanity_check_time_skew()
-#
-# Revision 1.17  2008/03/11 17:00:49  ncq
-# - explicitely request readonly raw connection
-#
-# Revision 1.16  2008/03/06 18:29:29  ncq
-# - standard lib logging only
-#
-# Revision 1.15  2008/03/05 22:26:52  ncq
-# - spelling
-#
-# Revision 1.14  2008/02/25 17:33:16  ncq
-# - use improved db sanity checks
-#
-# Revision 1.13  2008/01/30 14:07:02  ncq
-# - protect against faulty backend profile in preferences
-#
-# Revision 1.12  2008/01/27 21:13:17  ncq
-# - no more gmCfg
-#
-# Revision 1.11  2008/01/14 20:33:06  ncq
-# - cleanup
-# - properly detect connection errors
-#
-# Revision 1.10  2008/01/13 01:16:52  ncq
-# - log DSN on any connect errors
-#
-# Revision 1.9  2008/01/07 19:51:54  ncq
-# - bump db version
-# - we still need gmCfg
-#
-# Revision 1.8  2007/12/26 23:22:27  ncq
-# - fix invalid variable access
-#
-# Revision 1.7  2007/12/26 22:44:31  ncq
-# - missing import of logging
-# - use std lib logger
-# - cleanup
-#
-# Revision 1.6  2007/12/26 22:01:25  shilbert
-# - cleanup
-#
-# Revision 1.5  2007/12/23 22:02:56  ncq
-# - no more gmCLI
-#
-# Revision 1.4  2007/12/23 20:26:31  ncq
-# - cleanup
-#
-# Revision 1.3  2007/12/23 12:07:40  ncq
-# - cleanup
-# - use gmCfg2
-#
-# Revision 1.2  2007/12/04 16:14:52  ncq
-# - get_dbowner_connection()
-#
-# Revision 1.1  2007/12/04 16:03:43  ncq
-# - merger of gmLogin and gmLoginDialog
-#
-#
 
-#----------------------------------------------------------------
-# old logs
-#----------------------------------------------------------------
-# Log: gmLogin.py,v
-# Revision 1.37  2007/12/04 15:23:14  ncq
-# - reorder connect_to_database()
-# - check server settings sanity
-#
-# Revision 1.36  2007/11/09 14:40:33  ncq
-# - gmPG2 dumps schema by itself now on hash mismatch
-#
-# Revision 1.35  2007/10/23 21:24:39  ncq
-# - start backend listener on login
-#
-# Revision 1.34  2007/06/11 20:25:18  ncq
-# - better logging
-#
-# Revision 1.33  2007/04/27 13:29:31  ncq
-# - log schema dump on version conflict
-#
-# Revision 1.32  2007/03/18 14:10:07  ncq
-# - do show schema mismatch warning even if --override-schema-check
-#
-# Revision 1.31  2007/03/08 11:41:55  ncq
-# - set default client encoding from here
-# - save state when --override-schema-check == True, too
-#
-# Revision 1.30  2006/12/15 15:27:01  ncq
-# - move database schema version check here
-#
-# Revision 1.29  2006/10/25 07:46:44  ncq
-# - Format() -> strftime() since datetime.datetime does not have .Format()
-#
-# Revision 1.28  2006/10/25 07:21:57  ncq
-# - no more gmPG
-#
-# Revision 1.27  2006/10/24 13:25:19  ncq
-# - Login() -> connect_to_database()
-# - make gmPG2 main connection provider, piggyback gmPG onto it for now
-#
-# Revision 1.26  2006/10/08 11:04:45  ncq
-# - simplify wx import
-# - piggyback gmPG2 until gmPG is pruned
-#
-# Revision 1.25  2006/01/03 12:12:03  ncq
-# - make epydoc happy re _()
-#
-# Revision 1.24  2005/09/27 20:44:59  ncq
-# - wx.wx* -> wx.*
-#
-# Revision 1.23  2005/09/26 18:01:51  ncq
-# - use proper way to import wx26 vs wx2.4
-# - note: THIS WILL BREAK RUNNING THE CLIENT IN SOME PLACES
-# - time for fixup
-#
-# Revision 1.22  2004/09/13 08:54:49  ncq
-# - cleanup
-# - use gmGuiHelpers
-#
-# Revision 1.21  2004/07/18 20:30:54  ncq
-# - wxPython.true/false -> Python.True/False as Python tells us to do
-#
-# Revision 1.20  2004/06/20 16:01:05  ncq
-# - please epydoc more carefully
-#
-# Revision 1.19  2004/06/20 06:49:21  ihaywood
-# changes required due to Epydoc's OCD
-#
-# Revision 1.18  2004/03/04 19:47:06  ncq
-# - switch to package based import: from Gnumed.foo import bar
-#
-# Revision 1.17  2003/11/17 10:56:38  sjtan
-#
-# synced and commiting.
-#
-# Revision 1.1  2003/10/23 06:02:39  sjtan
-#
-# manual edit areas modelled after r.terry's specs.
-#
-# Revision 1.16  2003/06/26 21:40:29  ncq
-# - fatal->verbose
-#
-# Revision 1.15  2003/02/07 14:28:05  ncq
-# - cleanup, cvs keywords
-#
-# @change log:
-#	29.10.2001 hherb first draft, untested
-#
-#----------------------------------------------------------------
-# Log: gmLoginDialog.py,v
-# Revision 1.90  2007/10/22 12:38:32  ncq
-# - default db change
-#
-# Revision 1.89  2007/10/07 12:32:41  ncq
-# - workplace property now on gmSurgery.gmCurrentPractice() borg
-#
-# Revision 1.88  2007/09/20 19:07:38  ncq
-# - port 5432 on salaam again
-#
-# Revision 1.87  2007/09/04 23:30:28  ncq
-# - support --slave and slave mode checkbox
-#
-# Revision 1.86  2007/08/07 21:42:40  ncq
-# - cPaths -> gmPaths
-#
-# Revision 1.85  2007/06/14 21:55:49  ncq
-# - try to fix wx2.8 problem with size
-#
-# Revision 1.84  2007/06/11 20:25:55  ncq
-# - bump database version
-#
-# Revision 1.83  2007/06/10 10:17:54  ncq
-# - fix when no --debug
-#
-# Revision 1.82  2007/05/22 13:35:11  ncq
-# - default port 5433 now on salaam
-#
-# Revision 1.81  2007/05/08 11:16:10  ncq
-# - set debugging flag based on user checkbox selection
-#
-# Revision 1.80  2007/05/07 12:33:31  ncq
-# - use gmTools.cPaths
-# - support debug mode checkbox
-#
-# Revision 1.79  2007/04/11 20:45:01  ncq
-# - no more 'resource dir'
-#
-# Revision 1.78  2007/04/02 15:15:19  ncq
-# - v5 -> v6
-#
-# Revision 1.77  2007/03/08 16:20:50  ncq
-# - need to reference proper cfg file
-#
-# Revision 1.76  2007/03/08 11:46:23  ncq
-# - restructured
-#   - cleanup
-#   - no more cLoginParamChoices
-#   - no more loginparams keyword
-#   - properly set user preferences file
-#   - backend profile now can contain per-profile default encoding
-#   - support system-wide profile pre-defs in /etc/gnumed/gnumed-client.conf
-#
-# Revision 1.75  2007/02/17 14:13:11  ncq
-# - gmPerson.gmCurrentProvider().workplace now property
-#
-# Revision 1.74  2007/01/30 17:40:22  ncq
-# - cleanup, get rid of wxMAC IFDEF
-# - use user preferences file for storing login preferences
-#
-# Revision 1.73  2006/12/21 16:54:32  ncq
-# - inage handlers already inited
-#
-# Revision 1.72  2006/12/15 15:27:28  ncq
-# - cleanup
-#
-# Revision 1.71  2006/10/25 07:21:57  ncq
-# - no more gmPG
-#
-# Revision 1.70  2006/09/01 14:45:03  ncq
-# - assume conf file is latin1 ... FIX later !
-#
-# Revision 1.69  2006/07/30 17:50:03  ncq
-# - properly load bitmaps
-#
-# Revision 1.68  2006/05/15 07:05:07  ncq
-# - must import gmPerson now
-#
-# Revision 1.67  2006/05/14 21:44:22  ncq
-# - add get_workplace() to gmPerson.gmCurrentProvider and make use thereof
-# - remove use of gmWhoAmI.py
-#
-# Revision 1.66  2006/05/12 12:18:11  ncq
-# - whoami -> whereami cleanup
-# - use gmCurrentProvider()
-#
-# Revision 1.65  2005/09/28 21:27:30  ncq
-# - a lot of wx2.6-ification
-#
-# Revision 1.64  2005/09/27 20:44:59  ncq
-# - wx.wx* -> wx.*
-#
-# Revision 1.63  2005/09/26 18:01:51  ncq
-# - use proper way to import wx26 vs wx2.4
-# - note: THIS WILL BREAK RUNNING THE CLIENT IN SOME PLACES
-# - time for fixup
-#
-# Revision 1.62  2005/09/24 09:17:29  ncq
-# - some wx2.6 compatibility fixes
-#
-# Revision 1.61  2005/08/15 15:15:25  ncq
-# - improved error message
-#
-# Revision 1.60  2005/08/14 15:22:04  ncq
-# - need to import gmNull
-#
-# Revision 1.59  2005/08/14 14:31:53  ncq
-# - better handling of missing/malformed config file
-#
-# Revision 1.58  2005/06/10 16:09:36  shilbert
-# foo.AddSizer()-> foo.Add() such that wx2.5 works
-#
-# Revision 1.57  2005/03/06 14:54:19  ncq
-# - szr.AddWindow() -> Add() such that wx2.5 works
-# - 'demographic record' -> get_identity()
-#
-# Revision 1.56  2005/01/22 22:26:06  ncq
-# - a bunch of cleanups
-# - i18n
-#
-# Revision 1.55  2005/01/20 21:37:40  hinnef
-# - added error dialog when no profiles specified
-#
-# Revision 1.54  2004/11/24 21:16:33  hinnef
-# - fixed issue with missing profiles when writing to empty gnumed.conf.
-# This lead to a crash when trying to load the invalid gnumed.conf.
-# Now we just ignore this and fall back to defaults.
-#
-# Revision 1.53  2004/11/22 19:32:36  hinnef
-# new login dialog with profiles
-#
-# Revision 1.52  2004/09/13 08:57:33  ncq
-# - losts of cleanup
-# - opt/tty not used in DSN anymore
-# - --slave -> gb['main.slave_mode']
-#
-# Revision 1.51  2004/07/18 20:30:54  ncq
-# - wxPython.true/false -> Python.True/False as Python tells us to do
-#
-# Revision 1.50  2004/07/18 18:44:27  ncq
-# - use Python True, not wxPython true
-#
-# Revision 1.49  2004/06/20 16:01:05  ncq
-# - please epydoc more carefully
-#
-# Revision 1.48  2004/06/20 06:49:21  ihaywood
-# changes required due to Epydoc's OCD
-#
-# Revision 1.47  2004/05/28 13:06:41  ncq
-# - improve faceName/Mac OSX fix
-#
-# Revision 1.46  2004/05/28 09:09:34  shilbert
-# - remove faceName option from wx.Font on wxMac or else no go
-#
-# Revision 1.45  2004/05/26 20:35:23  ncq
-# - don't inherit from LoginDialog in OptionWindow, it barks on MacOSX
-#
-# Revision 1.44  2004/04/24 12:46:35  ncq
-# - logininfo() needs host=
-#
-# Revision 1.43  2004/03/04 19:47:06  ncq
-# - switch to package based import: from Gnumed.foo import bar
-#
-# Revision 1.42  2003/12/29 16:58:52  uid66147
-# - use whoami
-#
-# Revision 1.41  2003/11/17 10:56:38  sjtan
-#
-# synced and commiting.
-#
-# Revision 1.1  2003/10/23 06:02:39  sjtan
-#
-# manual edit areas modelled after r.terry's specs.
-#
-# Revision 1.40  2003/06/17 19:27:44  ncq
-# - cleanup
-# - only use cfg file login params if not empty list
-#
-# Revision 1.39  2003/05/17 17:30:36  ncq
-# - just some cleanup
-#
-# Revision 1.38  2003/05/12 09:01:45  ncq
-# - 1) type(tmp) is types.ListType implies tmp != None
-# - 2) check for ListType, not StringType
-#
-# Revision 1.37  2003/05/10 18:48:09  hinnef
-# - added stricter type checks when reading from cfg-file
-#
-# Revision 1.36  2003/04/05 00:37:46  ncq
-# - renamed a few variables to reflect reality
-#
-# Revision 1.35  2003/03/31 00:18:34  ncq
-# - or so I thought
-#
-# Revision 1.34  2003/03/31 00:17:43  ncq
-# - started cleanup/clarification
-# - made saving of values work again
-#
-# Revision 1.33  2003/02/09 18:55:47  ncq
-# - make comment less angry
-#
-# Revision 1.32  2003/02/08 00:32:30  ncq
-# - fixed failure to detect config file
-#
-# Revision 1.31  2003/02/08 00:15:17  ncq
-# - cvs metadata keywords
-#
-# Revision 1.30  2003/02/07 21:06:02  sjtan
-#
-# refactored edit_area_gen_handler to handler_generator and handler_gen_editarea. New handler for gmSelectPerson
-#
-# Revision 1.29  2003/02/02 07:38:29  michaelb
-# set serpent as window icon - login dialog & option dialog
-#
-# Revision 1.28  2003/01/16 09:22:28  ncq
-# - changed default workplace name to "__default__" to play better with the database
-#
-# Revision 1.27  2003/01/07 10:22:52  ncq
-# - fixed font definition
-#
-# Revision 1.26  2002/09/21 14:49:22  ncq
-# - cleanup related to gmi18n
-#
-# Revision 1.25  2002/09/12 23:19:27  ncq
-# - cleanup in preparation of cleansing
-#
-# Revision 1.24  2002/09/10 08:54:35  ncq
-# - remember workplace name once loaded
-#
-# Revision 1.23  2002/09/09 01:05:16  ncq
-# - fixed i18n glitch
-# - added note that _("") is invalid !!
-#
-# @change log:
-#	10.10.2001 hherb initial implementation, untested
-#	24.10.2001 hherb comments added
-#	24.10.2001 hherb LoginDialog class added,
-#	24.10.2001 hherb persistent changes across multiple processes enabled
-#	25.10.2001 hherb more flexible configuration options, more commenting
-#	07.02.2002 hherb saved parameters now showing corectly in combo boxes
-#   	05.09.2002 hberger moved options to own dialog
-#       06.09.2002 rterry simplified gui
-#                  -duplication and visual clutter removed
-#                   removed duplication of information in display
-#                   login gone from title bar , occurs once only now
-#                   visually highlighted in dark blue
-#                   login button title now 'ok' to conform to usual defaults of 
-#                   ok, cancel in most operating systems
-#                   reference to help removed (have help button for that)
-#                   date removed - can't see use of this in login - clutters
-#   	06.09.2002 hberger removed old code, fixed "login on close window" bug
-#       07.09.2002 rterry removed duplicated heading in advanced login options
