@@ -1,9 +1,7 @@
 """GNUmed medication/substances handling widgets.
 """
 #================================================================
-# $Source: /home/ncq/Projekte/cvs2git/vcs-mirror/gnumed/gnumed/client/wxpython/gmMedicationWidgets.py,v $
-# $Id: gmMedicationWidgets.py,v 1.32 2010-01-13 21:52:04 ncq Exp $
-__version__ = "$Revision: 1.32 $"
+__version__ = "$Revision: 1.33 $"
 __author__ = "Karsten Hilbert <Karsten.Hilbert@gmx.net>"
 
 import logging, sys, os.path
@@ -19,10 +17,42 @@ from Gnumed.pycommon import gmMatchProvider, gmI18N, gmPrinting, gmCfg2
 from Gnumed.business import gmPerson, gmATC, gmSurgery, gmMedication, gmForms
 from Gnumed.wxpython import gmGuiHelpers, gmRegetMixin, gmAuthWidgets, gmEditArea, gmMacro
 from Gnumed.wxpython import gmCfgWidgets, gmListWidgets, gmPhraseWheel, gmFormWidgets
+from Gnumed.wxpython import gmAllergyWidgets
 
 
 _log = logging.getLogger('gm.ui')
 _log.info(__version__)
+#============================================================
+def browse_atc_reference(parent=None):
+
+	if parent is None:
+		parent = wx.GetApp().GetTopWindow()
+	#------------------------------------------------------------
+	def refresh(lctrl):
+		atcs = gmATC.get_reference_atcs()
+
+		items = [ [
+			a['atc'],
+			a['term'],
+			u'%s' % gmTools.coalesce(a['ddd'], u''),
+			gmTools.coalesce(a['unit'], u''),
+			gmTools.coalesce(a['administrative_route'], u''),
+			gmTools.coalesce(a['comment'], u''),
+			a['version'],
+			a['lang']
+		] for a in atcs ]
+		lctrl.set_string_items(items)
+		lctrl.set_data(atcs)
+	#------------------------------------------------------------
+	gmListWidgets.get_choices_from_list (
+		parent = parent,
+		msg = _('\nThe ATC codes as known to GNUmed.\n'),
+		caption = _('Showing ATC codes.'),
+		columns = [ u'ATC', _('Term'), u'DDD', _('Unit'), _(u'Route'), _('Comment'), _('Version'), _('Language') ],
+		single_selection = True,
+		refresh_callback = refresh
+	)
+
 #============================================================
 def manage_substances_in_brands(parent=None):
 
@@ -44,6 +74,7 @@ def manage_substances_in_brands(parent=None):
 			s['substance'],
 			gmTools.coalesce(s['atc_substance'], u''),
 			s['preparation'],
+			#gmTools.coalesce(s['external_code_brand'], u'', u'%%s [%s]' % s['external_code_type_brand']),
 			gmTools.coalesce(s['external_code_brand'], u''),
 			s['pk_substance_in_brand']
 		] for s in substs ]
@@ -89,7 +120,7 @@ def manage_branded_drugs(parent=None):
 			d['description'],
 			d['preparation'],
 			gmTools.coalesce(d['atc_code'], u''),
-			gmTools.coalesce(d['external_code'], u''),
+			gmTools.coalesce(d['external_code'], u'', u'%%s [%s]' % d['external_code_type']),
 			d['pk']
 		] for d in drugs ]
 		lctrl.set_string_items(items)
@@ -207,6 +238,9 @@ def jump_to_drug_database():
 	drug_db = get_drug_database()
 	if drug_db is None:
 		return
+	pat = gmPerson.gmCurrentPatient()
+	if pat.connected:
+		drug_db.patient = pat
 	drug_db.switch_to_frontend(blocking = False)
 #============================================================
 def jump_to_ifap(import_drugs=False):
@@ -313,7 +347,7 @@ def update_atc_reference_data():
 	return True
 
 #============================================================
-# current medication widgets
+# current substance intake handling
 #============================================================
 class cSubstanceSchedulePhraseWheel(gmPhraseWheel.cPhraseWheel):
 
@@ -328,6 +362,7 @@ LIMIT 50"""
 
 		mp = gmMatchProvider.cMatchProvider_SQL2(queries = query)
 		mp.setThresholds(1, 2, 4)
+		mp.word_separators = '[ \t=+&:@]+'
 		gmPhraseWheel.cPhraseWheel.__init__(self, *args, **kwargs)
 		self.SetToolTipString(_('The schedule for taking this substance.'))
 		self.matcher = mp
@@ -370,6 +405,13 @@ class cSubstancePhraseWheel(gmPhraseWheel.cPhraseWheel):
 	SELECT NULL, (coalesce(atc_code || ': ', '') || description) as subst
 	FROM ref.substance_in_brand
 	WHERE description %(fragment_condition)s
+) union (
+	SELECT NULL, (atc || ': ' || term) as subst
+	FROM ref.v_atc
+	WHERE
+		is_group_code IS FALSE
+			AND
+		term %(fragment_condition)s
 )
 order by subst
 limit 50"""
@@ -426,7 +468,59 @@ class cCurrentMedicationEAPnl(wxgCurrentMedicationEAPnl.wxgCurrentMedicationEAPn
 		# adjust phrasewheels
 
 		self._PRW_brand.add_callback_on_lose_focus(callback = self._on_leave_brand)
+	#----------------------------------------------------------------
+	def __refresh_allergies(self):
+		emr = gmPerson.gmCurrentPatient().get_emr()
 
+		state = emr.allergy_state
+		if state['last_confirmed'] is None:
+			confirmed = _('never')
+		else:
+			confirmed = state['last_confirmed'].strftime('%Y %B %d').decode(gmI18N.get_encoding())
+		msg = _(u'%s, last confirmed %s\n') % (state.state_string, confirmed)
+		msg += gmTools.coalesce(state['comment'], u'', _('Comment (%s): %%s\n') % state['modified_by'])
+		msg += u'\n'
+
+		for allergy in emr.get_allergies():
+			msg += u'%s (%s, %s): %s\n' % (
+				allergy['descriptor'],
+				allergy['l10n_type'],
+				gmTools.bool2subst(allergy['definite'], _('definite'), _('suspected'), u'?'),
+				gmTools.coalesce(allergy['reaction'], _('reaction not recorded'))
+			)
+
+		self._LBL_allergies.SetLabel(msg)
+	#----------------------------------------------------------------
+	def __refresh_brand_and_components(self):
+
+		if self._PRW_brand.GetData() is None:
+			self._TCTRL_brand_ingredients.SetValue(u'')
+			if self.data is None:
+				return
+			if self.data['pk_brand'] is None:
+				return
+			self._PRW_brand.SetText(self.data['brand'], self.data['pk_brand'])
+
+		brand = gmMedication.cBrandedDrug(aPK_obj = self._PRW_brand.GetData())
+
+		if self.data is None:
+			self._PRW_preparation.SetText(brand['preparation'], None)
+		else:
+			self._PRW_preparation.SetText (
+				gmTools.coalesce(self.data['preparation'], brand['preparation']),
+				self.data['preparation']
+			)
+
+		comps = brand.components
+
+		if comps is None:
+			return
+
+		if len(comps) == 0:
+			return
+
+		comps = u' / '.join([ u'%s%s' % (c['description'], gmTools.coalesce(c['atc_code'], u'', u' (%s)')) for c in comps ])
+		self._TCTRL_brand_ingredients.SetValue(comps)
 	#----------------------------------------------------------------
 	# generic Edit Area mixin API
 	#----------------------------------------------------------------
@@ -465,6 +559,17 @@ class cCurrentMedicationEAPnl(wxgCurrentMedicationEAPnl.wxgCurrentMedicationEAPn
 				else:
 					self._PRW_duration.display_as_valid(True)
 
+		end = self._DP_discontinued.GetValue(as_pydt = True, invalid_as_none = True)
+		if end is not None:
+			start = self._DP_started.GetValue(as_pydt = True)
+			if start > end:
+				self._DP_started.display_as_valid(False)
+				self._DP_discontinued.display_as_valid(False)
+				validity = False
+			else:
+				self._DP_started.display_as_valid(True)
+				self._DP_discontinued.display_as_valid(True)
+
 		if validity is False:
 			gmDispatcher.send(signal = 'statustext', msg = _('Cannot save substance intake. Invalid or missing essential input.'))
 
@@ -488,7 +593,12 @@ class cCurrentMedicationEAPnl(wxgCurrentMedicationEAPnl.wxgCurrentMedicationEAPn
 		)
 
 		intake['strength'] = self._PRW_strength.GetValue()
-		intake['started'] = gmDateTime.wxDate2py_dt(wxDate = self._DP_started.GetValue())
+		intake['started'] = self._DP_started.GetValue(as_pydt = True, invalid_as_none = True)
+		intake['discontinued'] = self._DP_discontinued.GetValue(as_pydt = True, invalid_as_none = True)
+		if intake['discontinued'] is None:
+			intake['discontinue_reason'] = None
+		else:
+			intake['discontinue_reason'] = self._PRW_discontinue_reason().GetValue().strip()
 		intake['schedule'] = self._PRW_schedule.GetValue()
 		intake['aim'] = self._PRW_aim.GetValue()
 		intake['notes'] = self._PRW_notes.GetValue()
@@ -542,6 +652,13 @@ class cCurrentMedicationEAPnl(wxgCurrentMedicationEAPnl.wxgCurrentMedicationEAPn
 				brand.add_component(substance = subst['description'])
 
 		self.data = intake
+
+		if self._CHBOX_is_allergy.IsChecked():
+			allg = self.data.turn_into_allergy(encounter_id = emr.active_encounter['pk_encounter'])
+			# open for editing
+			dlg = gmAllergyWidgets.cAllergyManagerDlg(parent = self, id = -1)
+			dlg.ShowModal()
+
 		return True
 	#----------------------------------------------------------------
 	def _save_as_update(self):
@@ -553,7 +670,12 @@ class cCurrentMedicationEAPnl(wxgCurrentMedicationEAPnl.wxgCurrentMedicationEAPn
 		else:
 			self.data['pk_substance'] = self._PRW_substance.GetData()
 
-		self.data['started'] = gmDateTime.wxDate2py_dt(wxDate = self._DP_started.GetValue())
+		self.data['started'] = self._DP_started.GetValue(as_pydt = True, invalid_as_none = True)
+		self.data['discontinued'] = self._DP_discontinued.GetValue(as_pydt = True, invalid_as_none = True)
+		if self.data['discontinued'] is None:
+			self.data['discontinue_reason'] = None
+		else:
+			self.data['discontinue_reason'] = self._PRW_discontinue_reason.GetValue().strip()
 		self.data['preparation'] = self._PRW_preparation.GetValue()
 		self.data['strength'] = self._PRW_strength.GetValue()
 		self.data['schedule'] = self._PRW_schedule.GetValue()
@@ -581,12 +703,19 @@ class cCurrentMedicationEAPnl(wxgCurrentMedicationEAPnl.wxgCurrentMedicationEAPn
 			self.data['pk_brand'] = self._PRW_brand.GetData()
 
 		self.data.save()
+
+		if self._CHBOX_is_allergy.IsChecked():
+			allg = self.data.turn_into_allergy(encounter_id = emr.active_encounter['pk_encounter'])
+			# open for editing
+			dlg = gmAllergyWidgets.cAllergyManagerDlg(parent = self, id = -1)
+			dlg.ShowModal()
+
 		return True
 	#----------------------------------------------------------------
 	def _refresh_as_new(self):
 		self._PRW_substance.SetText(u'', None)
 		self._PRW_strength.SetText(u'', None)
-		self._PRW_preparation.SetText(u'', None)
+#		self._PRW_preparation.SetText(u'', None)
 		self._PRW_schedule.SetText(u'', None)
 		self._PRW_duration.SetText(u'', None)
 		self._PRW_aim.SetText(u'', None)
@@ -596,10 +725,12 @@ class cCurrentMedicationEAPnl(wxgCurrentMedicationEAPnl.wxgCurrentMedicationEAPn
 		self._CHBOX_long_term.SetValue(False)
 		self._CHBOX_approved.SetValue(True)
 
-		self._DP_started.SetValue(dt = gmDateTime.py_dt2wxDate(py_dt = gmDateTime.pydt_now_here(), wx = wx))
+		self._DP_started.SetValue(gmDateTime.pydt_now_here())
+		self._DP_discontinued.SetValue(None)
+		self._PRW_discontinue_reason.SetValue(u'')
 
-		self._PRW_brand.SetText(u'', None)
-		self._TCTRL_brand_ingredients.SetValue(u'')
+		self.__refresh_brand_and_components()
+		self.__refresh_allergies()
 
 		self._PRW_substance.SetFocus()
 	#----------------------------------------------------------------
@@ -607,14 +738,16 @@ class cCurrentMedicationEAPnl(wxgCurrentMedicationEAPnl.wxgCurrentMedicationEAPn
 
 		self._PRW_substance.SetText(self.data['substance'], self.data['pk_substance'])
 		self._PRW_strength.SetText(gmTools.coalesce(self.data['strength'], u''), self.data['strength'])
-		self._PRW_preparation.SetText(gmTools.coalesce(self.data['preparation'], u''), self.data['preparation'])
+
 		if self.data['is_long_term']:
 	 		self._CHBOX_long_term.SetValue(True)
 			self._PRW_duration.Enable(False)
 			self._PRW_duration.SetText(gmTools.u_infinity, None)
+			self._BTN_discontinued_as_planned.Enable(False)
 		else:
 			self._CHBOX_long_term.SetValue(False)
 			self._PRW_duration.Enable(True)
+			self._BTN_discontinued_as_planned.Enable(True)
 			if self.data['duration'] is None:
 				self._PRW_duration.SetText(u'', None)
 			else:
@@ -626,17 +759,12 @@ class cCurrentMedicationEAPnl(wxgCurrentMedicationEAPnl.wxgCurrentMedicationEAPn
 
 		self._CHBOX_approved.SetValue(self.data['intake_is_approved_of'])
 
-		self._DP_started.SetValue(gmDateTime.py_dt2wxDate(py_dt = self.data['started'], wx = wx))
+		self._DP_started.SetValue(self.data['started'])
+		self._DP_discontinued.SetValue(self.data['discontinued'])
+		self._PRW_discontinue_reason.SetValue(gmTools.coalesce(self.data['discontinue_reason'], u''))
 
-		self._PRW_brand.SetText(u'', None)
-		self._TCTRL_brand_ingredients.SetValue(u'')
-		if self.data['pk_brand'] is not None:
-			brand = gmMedication.cBrandedDrug(aPK_obj = self.data['pk_brand'])
-			self._PRW_brand.SetText(brand['description'], self.data['pk_brand'])
-			comps = brand.components
-			if comps is not None:
-				comps = u' / '.join([ u'%s%s' % (c['description'], gmTools.coalesce(c['atc_code'], u'', u' (%s)')) for c in comps ])
-				self._TCTRL_brand_ingredients.SetValue(comps)
+		self.__refresh_brand_and_components()
+		self.__refresh_allergies()
 
 		self._PRW_substance.SetFocus()
 	#----------------------------------------------------------------
@@ -647,36 +775,37 @@ class cCurrentMedicationEAPnl(wxgCurrentMedicationEAPnl.wxgCurrentMedicationEAPn
 		self._PRW_strength.SetText(u'', None)
 		self._PRW_notes.SetText(u'', None)
 
+		self.__refresh_brand_and_components()
+		self.__refresh_allergies()
+
 		self._PRW_substance.SetFocus()
 	#----------------------------------------------------------------
 	# event handlers
 	#----------------------------------------------------------------
 	def _on_leave_brand(self):
-		self._TCTRL_brand_ingredients.SetValue(u'')
-
-		pk_brand = self._PRW_brand.GetData()
-		if pk_brand is None:
-			return
-
-		brand = gmMedication.cBrandedDrug(aPK_obj = pk_brand)
-		self._PRW_preparation.SetText(brand['preparation'], None)
-
-		comps = brand.components
-		if comps is None:
-			return
-		comps = u' / '.join([ u'%s%s' % (c['description'], gmTools.coalesce(c['atc_code'], u'', u' (%s)')) for c in comps ])
-		self._TCTRL_brand_ingredients.SetValue(comps)
+		self.__refresh_brand_and_components()
+	#----------------------------------------------------------------
+	def _on_discontinued_date_changed(self, event):
+		if self._DP_discontinued.GetValue() is None:
+			self._PRW_discontinue_reason.Enable(False)
+			self._CHBOX_is_allergy.Enable(False)
+		else:
+			self._PRW_discontinue_reason.Enable(True)
+			self._CHBOX_is_allergy.Enable(True)
 	#----------------------------------------------------------------
 	def _on_get_substance_button_pressed(self, event):
 		drug_db = get_drug_database()
 		if drug_db is None:
 			return
+
 		result = drug_db.import_drugs()
 		if result is None:
 			return
+
 		new_drugs, new_substances = result
 		if len(new_substances) == 0:
 			return
+
 		# FIXME: could usefully
 		# FIXME: a) ask which to post-process
 		# FIXME: b) remember the others for post-processing
@@ -685,11 +814,15 @@ class cCurrentMedicationEAPnl(wxgCurrentMedicationEAPnl.wxgCurrentMedicationEAPn
 	#----------------------------------------------------------------
 	def _on_get_brand_button_pressed(self, event):
 		drug_db = get_drug_database()
+		self.__refresh_allergies()
 		if drug_db is None:
 			return
+
 		result = drug_db.import_drugs()
+		self.__refresh_allergies()
 		if result is None:
 			return
+
 		new_drugs, new_substances = result
 		if len(new_drugs) == 0:
 			return
@@ -698,12 +831,59 @@ class cCurrentMedicationEAPnl(wxgCurrentMedicationEAPnl.wxgCurrentMedicationEAPn
 		# FIXME: b) remember the others for post-processing
 		first = new_drugs[0]
 		self._PRW_brand.SetText(first['description'], first['pk'])
+
+		self.__refresh_brand_and_components()
+	#----------------------------------------------------------------
+	def _on_discontinued_as_planned_button_pressed(self, event):
+
+		now = gmDateTime.pydt_now_here()
+
+		self.__refresh_allergies()
+
+		# do we have a (full) plan ?
+		if None not in [self.data['started'], self.data['duration']]:
+			planned_end = self.data['started'] + self.data['duration']
+			# the plan hasn't ended so [Per plan] can't apply ;-)
+			if planned_end > now:
+				return
+			self._DP_discontinued.SetValue(planned_end)
+			self._PRW_discontinue_reason.Enable(True)
+			self._PRW_discontinue_reason.SetValue(u'')
+			self._CHBOX_is_allergy.Enable(True)
+			return
+
+		# we know started but not duration: apparently the plan is to stop today
+		if self.data['started'] is not None:
+			# but we haven't started yet so we can't stop
+			if self.data['started'] > now:
+				return
+
+		self._DP_discontinued.SetValue(now)
+		self._PRW_discontinue_reason.Enable(True)
+		self._PRW_discontinue_reason.SetValue(u'')
+		self._CHBOX_is_allergy.Enable(True)
 	#----------------------------------------------------------------
 	def _on_chbox_long_term_checked(self, event):
 		if self._CHBOX_long_term.IsChecked() is True:
 			self._PRW_duration.Enable(False)
+			self._BTN_discontinued_as_planned.Enable(False)
+			self._PRW_discontinue_reason.Enable(False)
+			self._CHBOX_is_allergy.Enable(False)
 		else:
 			self._PRW_duration.Enable(True)
+			self._BTN_discontinued_as_planned.Enable(True)
+			self._PRW_discontinue_reason.Enable(True)
+			self._CHBOX_is_allergy.Enable(True)
+
+		self.__refresh_allergies()
+	#----------------------------------------------------------------
+	def _on_chbox_is_allergy_checked(self, event):
+		if self._CHBOX_is_allergy.IsChecked() is True:
+			val = self._PRW_discontinue_reason.GetValue().strip()
+			if not val.startswith(_('not tolerated:')):
+				self._PRW_discontinue_reason.SetValue(u'%s %s' % (_('not tolerated:'), val))
+
+		self.__refresh_allergies()
 #============================================================
 def delete_substance_intake(parent=None, substance=None):
 	delete_it = gmGuiHelpers.gm_show_question (
@@ -744,6 +924,10 @@ def configure_medication_list_template(parent=None):
 		gmDispatcher.send(signal = 'statustext', msg = _('No medication list template configured.'), beep = True)
 		return None
 
+	if template['engine'] != u'L':
+		gmDispatcher.send(signal = 'statustext', msg = _('No medication list template configured.'), beep = True)
+		return None
+
 	dbcfg = gmCfg.cCfgSQL()
 	dbcfg.set (
 		workplace = gmSurgery.gmCurrentPractice().active_workplace,
@@ -792,7 +976,16 @@ def print_medication_list(parent=None, cleanup=True):
 			return False
 
 	# 2) process template
-	meds_list = template.instantiate()
+	try:
+		meds_list = template.instantiate()
+	except KeyError:
+		_log.exception('cannot instantiate medication list template [%s]', template)
+		gmGuiHelpers.gm_show_error (
+			aMessage = _('Invalid medication list template [%s - %s (%s)]') % (name, ver, template['engine']),
+			aTitle = _('Printing medication list')
+		)
+		return False
+
 	ph = gmMacro.gmPlaceholderHandler()
 	#ph.debug = True
 	meds_list.substitute_placeholders(data_source = ph)
@@ -960,7 +1153,21 @@ class cCurrentSubstancesGrid(wx.grid.Grid):
 			med = meds[row_idx]
 			self.__row_data[row_idx] = med
 
-			if med['is_currently_active'] is not True:
+			if med['is_currently_active'] is True:
+				atcs = []
+				if med['atc_substance'] is not None:
+					atcs.append(med['atc_substance'])
+				if med['atc_brand'] is not None:
+					atcs.append(med['atc_brand'])
+				allg = emr.is_allergic_to(atcs = tuple(atcs), inns = (med['substance'],), brand = med['brand'])
+				if allg not in [None, False]:
+					attr = self.GetOrCreateCellAttr(row_idx, 0)
+					if allg['type'] == u'allergy':
+						attr.SetTextColour('red')
+					else:
+						attr.SetTextColour('yellow')
+					self.SetRowAttr(row_idx, attr)
+			else:
 				attr = self.GetOrCreateCellAttr(row_idx, 0)
 				attr.SetTextColour('grey')
 				self.SetRowAttr(row_idx, attr)
@@ -1113,6 +1320,28 @@ class cCurrentSubstancesGrid(wx.grid.Grid):
 		subst = self.get_selected_data()[0]
 		delete_substance_intake(parent = self, substance = subst['pk_substance_intake'])
 	#------------------------------------------------------------
+	def create_allergy_from_substance(self):
+		rows = self.get_selected_rows()
+
+		if len(rows) == 0:
+			return
+
+		if len(rows) > 1:
+			gmDispatcher.send(signal = 'statustext', msg = _('Cannot create allergy from more than one substance at once.'), beep = True)
+			return
+
+		subst = self.get_selected_data()[0]
+		if subst['is_currently_active']:
+			subst['discontinued'] = gmDateTime.pydt_now_here()
+		if subst['discontinue_reason'] is None:
+			subst['discontinue_reason'] = _('discontinued due to allergy or intolerance')
+		subst.save()
+
+		emr = self.__patient.get_emr()
+		allg = subst.turn_into_allergy(encounter_id = emr.active_encounter['pk_encounter'])
+		dlg = gmAllergyWidgets.cAllergyManagerDlg(parent = self, id = -1)
+		dlg.ShowModal()
+	#------------------------------------------------------------
 	def print_medication_list(self):
 		# there could be some filtering/user interaction going on here
 		_cfg = gmCfg2.gmCfgData()
@@ -1125,15 +1354,56 @@ class cCurrentSubstancesGrid(wx.grid.Grid):
 		except KeyError:
 			return u' '
 
+		emr = self.__patient.get_emr()
+		atcs = []
+		if entry['atc_substance'] is not None:
+			atcs.append(entry['atc_substance'])
+		if entry['atc_brand'] is not None:
+			atcs.append(entry['atc_brand'])
+		allg = emr.is_allergic_to(atcs = tuple(atcs), inns = (entry['substance'],), brand = entry['brand'])
+
 		tt = _('Substance intake entry (%s, %s)   [#%s]                     \n') % (
-			gmTools.bool2subst(entry['is_currently_active'], _('active'), _('inactive')),
-			gmTools.bool2subst(entry['intake_is_approved_of'], _('approved'), _('unapproved')),
+			gmTools.bool2subst (
+				boolean = entry['is_currently_active'],
+				true_return = gmTools.bool2subst (
+					boolean = entry['seems_inactive'],
+					true_return = _('active, needs check'),
+					false_return = _('active'),
+					none_return = _('assumed active')
+				),
+				false_return = _('inactive')
+			),
+			gmTools.bool2subst (
+				boolean = entry['intake_is_approved_of'],
+				true_return = _('approved'),
+				false_return = _('unapproved')
+			),
 			entry['pk_substance_intake']
 		)
 
-		tt += u' ' + _('Substance name: %s   [#%s]\n') % (entry['substance'], entry['pk_substance'])
+		if allg not in [None, False]:
+			certainty = gmTools.bool2subst(allg['definite'], _('definite'), _('suspected'))
+			tt += u'\n'
+			tt += u' !! ---- Cave ---- !!\n'
+			tt += u' %s (%s): %s (%s)\n' % (
+				allg['l10n_type'],
+				certainty,
+				allg['descriptor'],
+				gmTools.coalesce(allg['reaction'], u'')[:40]
+			)
+			tt += u'\n'
+
+		tt += u' ' + _('Substance: %s   [#%s]\n') % (entry['substance'], entry['pk_substance'])
 		tt += u' ' + _('Preparation: %s\n') % entry['preparation']
-		tt += gmTools.coalesce(entry['strength'], u'', _(' Amount per dose: %s\n'))
+		if entry['strength'] is not None:
+			tt += u' ' + _('Amount per dose: %s') % entry['strength']
+			if entry.ddd is not None:
+				tt += u' (DDD: %s %s)' % (entry.ddd['ddd'], entry.ddd['unit'])
+			tt += u'\n'
+		else:
+			if entry.ddd is not None:
+				tt += u' DDD: %s %s' % (entry.ddd['ddd'], entry.ddd['unit'])
+				tt += u'\n'
 		tt += gmTools.coalesce(entry['atc_substance'], u'', _(' ATC (substance): %s\n'))
 
 		tt += u'\n'
@@ -1163,6 +1433,12 @@ class cCurrentSubstancesGrid(wx.grid.Grid):
 			gmTools.bool2subst(entry['is_long_term'], _(' (long-term)'), _(' (short-term)'), u'')
 		)
 
+		if entry['discontinued'] is not None:
+			tt += _(' Discontinued %s\n') % (
+				entry['discontinued'].strftime('%Y %B %d').decode(gmI18N.get_encoding()),
+			)
+			tt += _(' Reason: %s\n') % entry['discontinue_reason']
+
 		tt += u'\n'
 
 		tt += gmTools.coalesce(entry['aim'], u'', _(' Aim: %s\n'))
@@ -1171,9 +1447,8 @@ class cCurrentSubstancesGrid(wx.grid.Grid):
 
 		tt += u'\n'
 
-#		tt += _(u'Revisions: %(row_ver)s, last %(mod_when)s by %(mod_by)s.') % ({
-		tt += _(u'Revisions: Last modified %(mod_when)s by %(mod_by)s.') % ({
-			#'row_ver': entry['row_version'],
+		tt += _(u'Revision: #%(row_ver)s, %(mod_when)s by %(mod_by)s.') % ({
+			'row_ver': entry['row_version'],
 			'mod_when': entry['modified_when'].strftime('%c').decode(gmI18N.get_encoding()),
 			'mod_by': entry['modified_by']
 		})
@@ -1346,138 +1621,27 @@ class cCurrentSubstancesPnl(wxgCurrentSubstancesPnl.wxgCurrentSubstancesPnl, gmR
 	#--------------------------------------------------------
 	def _on_print_button_pressed(self, event):
 		self._grid_substances.print_medication_list()
+	#--------------------------------------------------------
+	def _on_allergy_button_pressed(self, event):
+		self._grid_substances.create_allergy_from_substance()
 #============================================================
 # main
 #------------------------------------------------------------
 if __name__ == '__main__':
 
+	if len(sys.argv) < 2:
+		sys.exit()
+
+	if sys.argv[1] != 'test':
+		sys.exit()
+
 	from Gnumed.pycommon import gmI18N
 
 	gmI18N.activate_locale()
 	gmI18N.install_domain(domain = 'gnumed')
 
 	#----------------------------------------
-	#----------------------------------------
-	if (len(sys.argv) > 1) and (sys.argv[1] == 'test'):
-#		test_*()
-		pass
+#	test_*()
+	pass
 
 #============================================================
-# $Log: gmMedicationWidgets.py,v $
-# Revision 1.32  2010-01-13 21:52:04  ncq
-# - implement phrase wheel on schedule and preparation
-#
-# Revision 1.31  2010/01/09 20:15:06  ncq
-# - use Advice
-#
-# Revision 1.30  2010/01/08 12:21:04  ncq
-# - even better error detection on forms template loading
-#
-# Revision 1.29  2010/01/06 14:42:20  ncq
-# - medication list printing:
-# 	- tie cleanup to --debug
-# 	- better error detection
-#
-# Revision 1.28  2010/01/01 21:48:15  ncq
-# - remove confusing status message
-#
-# Revision 1.27  2009/12/26 19:08:38  ncq
-# - document printing of medication list in EMR
-#
-# Revision 1.26  2009/12/25 22:07:17  ncq
-# - cleanup
-# - configure-medication-list-template
-# - print-medication-list
-# - show-info-on-entry
-# - tooltips on substance entry rows
-#
-# Revision 1.25  2009/12/22 12:02:57  ncq
-# - cleanup
-#
-# Revision 1.24  2009/12/03 17:51:11  ncq
-# - explicit ATC col in brand component list
-#
-# Revision 1.23  2009/12/02 16:50:44  ncq
-# - enable brand component deletion
-# - normalize substance name before adding as component
-#
-# Revision 1.22  2009/12/01 21:55:24  ncq
-# - branded drug phrasewheel
-# - much improved substance intake EA implementation
-#
-# Revision 1.21  2009/11/30 13:15:20  ncq
-# - better meds grid column ordering as per list
-#
-# Revision 1.20  2009/11/29 20:01:46  ncq
-# - substance phrasewheel
-#
-# Revision 1.19  2009/11/29 16:05:41  ncq
-# - must set self.data *late* in _save-as-new or else !
-# - properly enable/disable duration PRW
-# - grid business logic moved into grid so no need to access self._grid_substances anymore
-#
-# Revision 1.18  2009/11/28 18:31:30  ncq
-# - implement drug brand management
-# - implement drug brand component management
-#
-# Revision 1.17  2009/11/24 20:59:59  ncq
-# - use import-drugs rather than import-drugs-as-substances
-# - improved grid layout as per list
-# - fix get-selected-data
-# - make grid wrapper do less
-#
-# Revision 1.16  2009/11/19 14:44:25  ncq
-# - improved plugin
-#
-# Revision 1.15  2009/11/15 01:09:07  ncq
-# - implement grouping/filtering
-# - mark unapproved vs approved if showing all substances
-#
-# Revision 1.14  2009/11/08 20:49:20  ncq
-# - implement deletion
-# - start grouping/filtering/ row tooltips
-#
-# Revision 1.13  2009/11/06 15:19:25  ncq
-# - implement saving as new substance intake
-#
-# Revision 1.12  2009/10/29 17:23:24  ncq
-# - consolidate get-drug-database
-# - much improved substance intake EA
-# - better naming and adjust to such
-#
-# Revision 1.11  2009/10/28 21:48:55  ncq
-# - further improved substances grid
-#
-# Revision 1.10  2009/10/28 16:43:42  ncq
-# - start implementing substances edit area
-# - enhance grid to allow actual substances management
-#
-# Revision 1.9  2009/10/26 22:30:58  ncq
-# - implement deletion of INN
-#
-# Revision 1.8  2009/10/21 20:41:53  ncq
-# - access MMI from substance management via NEW button
-#
-# Revision 1.7  2009/10/21 09:21:13  ncq
-# - manage substances
-# - jump to drug database
-#
-# Revision 1.6  2009/10/20 10:27:35  ncq
-# - implement configuration of drug data source
-#
-# Revision 1.5  2009/09/01 22:36:08  ncq
-# - add jump-to-mmi
-#
-# Revision 1.4  2009/06/20 12:46:04  ncq
-# - move IFAP handling here
-#
-# Revision 1.3  2009/06/10 21:02:34  ncq
-# - update-atc-reference-data
-#
-# Revision 1.2  2009/05/13 12:20:59  ncq
-# - improve and streamline
-#
-# Revision 1.1  2009/05/12 12:04:01  ncq
-# - substance intake handling
-#
-#
\ No newline at end of file
